import { CommonModule } from '@angular/common';
import { ɵɵdefineInjectable, Injectable, Component, ViewEncapsulation, ChangeDetectionStrategy, ElementRef, Optional, Self, Attribute, ChangeDetectorRef, Input, ViewChild, Pipe, NgModule } from '@angular/core';
import { NgForm, FormGroupDirective, NgControl, FormsModule } from '@angular/forms';
import { coerceBooleanProperty, coerceNumberProperty } from '@angular/cdk/coercion';
import { mixinTabIndex, mixinDisabled, mixinErrorState, ErrorStateMatcher } from '@angular/material/core';

Number.prototype.toFixedNoRounding = function (fractionDigits = 0) {
    const valueStr = this.toString();
    const commaIndex = valueStr.indexOf('.');
    if (commaIndex !== -1) {
        return +valueStr.substr(0, commaIndex + fractionDigits + 1);
    }
    return this;
};
var Round;
(function (Round) {
    Round[Round["up"] = 0] = "up";
    Round[Round["down"] = 1] = "down";
})(Round || (Round = {}));
class SimpleDecimal {
    constructor(maxDecimalPlaces, value = null) {
        this._maxDecimalPlacesFn = maxDecimalPlaces instanceof Function ? maxDecimalPlaces : () => maxDecimalPlaces;
        this.value = value;
    }
    get integerPart() {
        if (this._integerPart == null) {
            const valueStr = this.toString();
            const commaIndex = valueStr.indexOf('.');
            const integerPart = +valueStr.substr(0, commaIndex);
            if (commaIndex !== -1) {
                this._integerPart = integerPart;
            }
            else {
                this._integerPart = this.value;
            }
        }
        return this._integerPart;
    }
    get decimalPart() {
        const valueStr = this.toString();
        const commaIndex = valueStr.indexOf('.');
        if (commaIndex !== -1) {
            if (this.value >= 0) {
                return +('0.' + valueStr.substr(commaIndex + 1, this.maxDecimalPlaces));
            }
            return -('0.' + valueStr.substr(commaIndex + 1, this.maxDecimalPlaces));
        }
        return 0;
    }
    get dividend() {
        this.calculateFraction();
        return this._dividend;
    }
    get divisor() {
        this.calculateFraction();
        return this._divisor;
    }
    get decimalPlaces() {
        const valueStr = this.toString();
        const commaIndex = valueStr.indexOf('.');
        return commaIndex !== -1 ? valueStr.length - commaIndex : 0;
    }
    get value() {
        return this._value;
    }
    set value(value) {
        this._value = null;
        if (value != null) {
            this._value = +Number(value).toFixedNoRounding(this.maxDecimalPlaces);
        }
        this._integerPart = null;
        this._dividend = null;
        this._divisor = null;
    }
    get maxDecimalPlaces() {
        return this._maxDecimalPlacesFn();
    }
    get hasPrettyFraction() {
        if (this.dividend && this.divisor) {
            return true;
        }
        return false;
    }
    get precedingStr() {
        const integerPart = this.integerPart;
        return '' + (integerPart !== 0 ? integerPart : (this.value < 0 ? '-' : ''));
    }
    get dividendStr() {
        return '' + this.dividend;
    }
    get divisorStr() {
        return '' + this.divisor;
    }
    toString() {
        if (this.value != null) {
            return String(this.value).replace(',', '.');
        }
        else {
            return '';
        }
    }
    add(i) {
        this.value += i;
        return this;
    }
    sub(i) {
        this.value -= i;
        return this;
    }
    toNearest(multiple, roundMode = Round.up) {
        multiple = +multiple.toFixed(this.maxDecimalPlaces);
        if (multiple > 0 && this.value != null) {
            let quotient = +(this.decimalPart / multiple).toFixed(this.maxDecimalPlaces);
            switch (roundMode) {
                case Round.down:
                    quotient = Math.ceil(quotient) - 1;
                    break;
                default:
                    quotient = Math.floor(quotient) + 1;
                    break;
            }
            return +(this.integerPart + (multiple * quotient)).toFixed(this.maxDecimalPlaces);
        }
        return this.value;
    }
    calculateFraction() {
        if (this.maxDecimalPlaces >= 2 && !(this._dividend && this._divisor)) {
            const decimalPart = Math.abs(this.decimalPart);
            if (decimalPart) {
                switch (decimalPart) {
                    case (1 / 4).toFixedNoRounding(this.maxDecimalPlaces):
                        this._dividend = 1;
                        this._divisor = 4;
                        break;
                    case (1 / 3).toFixedNoRounding(this.maxDecimalPlaces):
                        this._dividend = 1;
                        this._divisor = 3;
                        break;
                    case (1 / 2).toFixedNoRounding(this.maxDecimalPlaces):
                        this._dividend = 1;
                        this._divisor = 2;
                        break;
                    case (2 / 3).toFixedNoRounding(this.maxDecimalPlaces):
                    case +(2 / 3).toFixed(this.maxDecimalPlaces):
                        this._dividend = 2;
                        this._divisor = 3;
                        break;
                    case (3 / 4).toFixedNoRounding(this.maxDecimalPlaces):
                        this._dividend = 3;
                        this._divisor = 4;
                        break;
                }
            }
        }
    }
}

var Direction;
(function (Direction) {
    Direction[Direction["default"] = 0] = "default";
    Direction[Direction["leftToRight"] = 1] = "leftToRight";
    Direction[Direction["rightToLeft"] = 2] = "rightToLeft";
})(Direction || (Direction = {}));

class DeviceService {
    constructor() {
        this.mobileOrTablet = false;
        this.mobileOrTablet = this.isMobileOrTablet();
    }
    isMobileOrTablet() {
        let check = false;
        (a => {
            // eslint-disable-next-line max-len
            if (/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino|android|ipad|playbook|silk/i.test(a) || /1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(a.substr(0, 4))) {
                check = true;
            }
            // eslint-disable-next-line @typescript-eslint/dot-notation
        })(navigator.userAgent || navigator.vendor || window['opera']);
        return check;
    }
}
DeviceService.ɵprov = ɵɵdefineInjectable({ factory: function DeviceService_Factory() { return new DeviceService(); }, token: DeviceService, providedIn: "root" });
DeviceService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
DeviceService.ctorParameters = () => [];

class LanguageService {
    get preferredLanguage() {
        if (this._language !== undefined) {
            return this._language;
        }
        const nav = window.navigator;
        const browserLanguagePropertyKeys = ['language', 'browserLanguage', 'systemLanguage', 'userLanguage'];
        // support for HTML 5.1 "navigator.languages"
        if (nav.languages instanceof Array) {
            for (const language of nav.languages) {
                if (language && language.length) {
                    return this._language = String(language).substring(0, 2).toLowerCase();
                }
            }
        }
        // support for other well known properties in browsers
        for (const prop of browserLanguagePropertyKeys) {
            const language = nav[prop];
            if (language && language.length) {
                return this._language = String(language).substring(0, 2).toLowerCase();
            }
        }
        return this._language = null;
    }
    isRTLpreferred() {
        if (this._isRTLpreferred !== undefined) {
            return this._isRTLpreferred;
        }
        switch (this.preferredLanguage) {
            case 'ar':
            case 'dv':
            case 'fa':
            case 'ha':
            case 'he':
            case 'ks':
            case 'ku':
            case 'ms':
            case 'ps':
            case 'sd':
            case 'tg':
            case 'ug':
            case 'ur':
            case 'uz':
            case 'yi':
                return this._isRTLpreferred = true;
            default:
                return this._isRTLpreferred = false;
        }
    }
}
LanguageService.ɵprov = ɵɵdefineInjectable({ factory: function LanguageService_Factory() { return new LanguageService(); }, token: LanguageService, providedIn: "root" });
LanguageService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];

class SpinnerKeyManager {
    constructor(spinner) {
        this.spinner = spinner;
    }
    catchKey(event) {
        const key = event.key;
        if (key) {
            switch (key) {
                case 'ArrowUp':
                    this.spinner.add(true);
                    break;
                case 'ArrowDown':
                    this.spinner.subtract(true);
                    break;
                default:
                    return;
            }
            // Prevent only for keys, because of a firefox css :active bug on buttons
            event.preventDefault();
        }
    }
}

let nextUniqueId = 0;
const BUFFER_TIMEOUT = 250;
const TIMEOUT = 50;
class SimpleSpinnerBase {
    constructor(_elementRef, _defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl) {
        this._elementRef = _elementRef;
        this._defaultErrorStateMatcher = _defaultErrorStateMatcher;
        this._parentForm = _parentForm;
        this._parentFormGroup = _parentFormGroup;
        this.ngControl = ngControl;
    }
}
// eslint-disable-next-line @typescript-eslint/naming-convention
const _SimpleSpinnerMixinBase = mixinTabIndex(mixinDisabled(mixinErrorState(SimpleSpinnerBase)));
class SpinnerComponent extends _SimpleSpinnerMixinBase {
    constructor(device, elementRef, _defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl, tabIndex, cdRef, langService) {
        super(elementRef, _defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl);
        this.device = device;
        this.ngControl = ngControl;
        this.cdRef = cdRef;
        this.langService = langService;
        this.hover = false;
        this.focus = false;
        this.intervalHandler = null;
        // Regex allows all numerals (0-9, Arabic, Hindi, Thai, ...) and commas
        // eslint-disable-next-line max-len
        this.filterKeyRegex = /^[0-9\u0660-\u0669\u06F0-\u06F9\u0966-\u096F\u0A66-\u0A6F\u0AE6-\u0AEF\u0B66-\u0B6F\u09E6-\u09EF\u0BE6-\u0BEF\u0C66-\u0C6F\u0CE6-\u0CEF\u0D66-\u0D6F\u0DE6-\u0DEF\u1040-\u1049\u17E0-\u17E9\u0E50-\u0E59\u0ED0-\u0ED9\u1810-\u1819\u0F20-\u0F29.,\-]+$/;
        // Enum for usage in template
        // eslint-disable-next-line @typescript-eslint/naming-convention
        this.Direction = Direction;
        this._min = Number.MIN_SAFE_INTEGER;
        this._max = Number.MAX_SAFE_INTEGER;
        this._hoverBorder = false;
        this._hoverNav = true;
        this._fractions = false;
        this._i18nNumbers = true;
        this._bigStep = 1;
        this._smallStep = 1;
        this._maxDecimalPlaces = 0;
        this._required = false;
        this.uid = `simple-spinner-${++nextUniqueId}`;
        this.propagateChange = (_) => { };
        this.propagateTouched = () => { };
        if (this.ngControl) {
            // Note: we provide the value accessor through here, instead of
            // the `providers` to avoid running into a circular import.
            // It has to be done this way also, to be able to access the errorState.
            this.ngControl.valueAccessor = this;
        }
        this.decimal = new SimpleDecimal(() => this.maxDecimalPlaces);
        this.keyManager = new SpinnerKeyManager(this);
        this.tabIndex = parseInt(tabIndex, 10) || 0;
        // Force setter to be called in case id was not specified.
        this.id = this.id;
    }
    get id() { return this._id; }
    set id(id) {
        this._id = id || this.uid;
    }
    get value() { return this.decimal.value; }
    set value(value) {
        this.decimal.value = value;
        // Return the decimal value instead of value, to propagate the rounded value
        this.propagateChange(this.decimal.value);
    }
    get min() { return this._min; }
    set min(min) {
        if (min != null) {
            min = parseFloat(min);
            if (!isNaN(min)) {
                this._min = Math.min(Math.max(min, Number.MIN_SAFE_INTEGER), Number.MAX_SAFE_INTEGER);
            }
        }
    }
    get max() { return this._max; }
    set max(max) {
        if (max != null) {
            max = parseFloat(max);
            if (!isNaN(max)) {
                this._max = Math.min(Math.max(max, Number.MIN_SAFE_INTEGER), Number.MAX_SAFE_INTEGER);
            }
        }
    }
    get hoverBorder() { return this._hoverBorder; }
    set hoverBorder(hoverBorder) {
        this._hoverBorder = coerceBooleanProperty(hoverBorder);
    }
    get hoverNav() { return this._hoverNav; }
    set hoverNav(hoverNav) {
        this._hoverNav = coerceBooleanProperty(hoverNav);
    }
    get fractions() { return this._fractions; }
    set fractions(fractions) {
        this._fractions = coerceBooleanProperty(fractions);
    }
    get i18nNumbers() { return this._i18nNumbers; }
    set i18nNumbers(i18nNumbers) {
        this._i18nNumbers = coerceBooleanProperty(i18nNumbers);
    }
    get bigStep() { return this._bigStep; }
    set bigStep(bigStep) {
        bigStep = coerceNumberProperty(bigStep);
        this._bigStep = Math.min(Math.max(bigStep, Number.MIN_SAFE_INTEGER), Number.MAX_SAFE_INTEGER);
    }
    get smallStep() { return this._smallStep; }
    set smallStep(smallStep) {
        smallStep = coerceNumberProperty(smallStep);
        this._smallStep = Math.min(Math.max(smallStep, Number.MIN_SAFE_INTEGER), Number.MAX_SAFE_INTEGER);
    }
    get maxDecimalPlaces() { return this._maxDecimalPlaces; }
    set maxDecimalPlaces(maxDecimalPlaces) {
        this._maxDecimalPlaces = coerceNumberProperty(maxDecimalPlaces);
    }
    get required() { return this._required; }
    set required(required) {
        this._required = coerceBooleanProperty(required);
    }
    get direction() {
        switch (this.dir) {
            case 'ltr':
                return Direction.leftToRight;
            case 'rtl':
                return Direction.rightToLeft;
            case 'auto':
                return this.langService.isRTLpreferred() ? Direction.rightToLeft : Direction.leftToRight;
            default:
                return Direction.default;
        }
    }
    get showBorder() {
        // Always show the border if disabled or focused
        return this.disabled || this.focus || this.errorState || !this.hoverBorder || this.hoverBorder && this.hover;
    }
    get showNav() {
        // Always show the navigation if disabled or focused
        // Never show on mobile
        return !this.device.mobileOrTablet && (this.disabled || this.focus || !this.hoverNav || this.hoverNav && this.hover);
    }
    get showFractionOverlay() {
        return this.decimal.hasPrettyFraction && this.fractions && !this.focus && (this.disabled || !this.hover);
    }
    get showPlaceholder() {
        return this.decimal.value == null && this.placeholder != null && !this.focus;
    }
    get ariaLabel() {
        return this._ariaLabel || this.placeholder;
    }
    checkValue() {
        // This has to be done over the ElementRef because Angular won't emit a change if the value stays the same
        // Also the value should only be submitted after blur
        const value = parseFloat(this.numberInput.nativeElement.value);
        this.value = isNaN(value) ? null : Math.min(Math.max(value, this.min), this.max);
        this.numberInput.nativeElement.value = this.value;
    }
    filterKey(event) {
        if (!event.ctrlKey &&
            !event.metaKey &&
            !event.altKey &&
            event.code !== '' && // On Chrome for Android the code is empty
            event.key !== event.code &&
            !event.key.match(this.filterKeyRegex)) {
            event.preventDefault();
        }
    }
    increment() {
        // Special logic for increment only, in case the value is null / undefined
        // It does not get more readable :(
        if (this.decimal.value == null) {
            if (this.min === Number.MIN_SAFE_INTEGER) {
                this.decimal.value = Math.min(this.max, this.bigStep);
            }
            else if (this.max === Number.MAX_SAFE_INTEGER) {
                this.decimal.value = Math.max(this.min, this.bigStep);
            }
            else {
                this.decimal.value = Math.min(this.max, this.bigStep);
            }
            this.propagateChange(this.decimal.value);
            this.cdRef.markForCheck();
            return;
        }
        // This might happen if the user sets the value lower than the min (same behaviour in Chrome)
        if (this.decimal.value < this.min) {
            this.decimal.value = this.min;
            this.propagateChange(this.decimal.value);
            this.cdRef.markForCheck();
            return;
        }
        if (this.decimal.value < this.max) {
            // If an integer and less or equal to max - 1
            if (this.decimal.decimalPlaces === 0 && this.decimal.value <= (this.max - this.bigStep)) {
                this.decimal.value += this.bigStep; // add
            }
            else { // Float value or greater than max - 1
                // Next closest value plus step: 0.6 -> 0.75 -> 1 ...
                const stepValue = this.decimal.toNearest(this.smallStep, Round.up);
                // If less or equal to max then set the value to the new stepValue
                if (stepValue <= this.max) {
                    this.decimal.value = stepValue;
                }
                else {
                    this.decimal.value = this.max;
                }
            }
            this.propagateChange(this.decimal.value);
            this.cdRef.markForCheck();
        }
        else {
            // Select value if max was reached
            this.numberInput.nativeElement.select();
        }
    }
    decrement() {
        // Special logic for decrement only, in case the value is null / undefined
        // It does not get more readable :(
        if (this.decimal.value == null) {
            if (this.max === Number.MAX_SAFE_INTEGER) {
                this.decimal.value = Math.max(this.min, -this.bigStep);
            }
            else if (this.min === Number.MIN_SAFE_INTEGER) {
                this.decimal.value = Math.min(this.max, -this.bigStep);
            }
            else {
                this.decimal.value = Math.max(this.min, -this.bigStep);
            }
            this.propagateChange(this.decimal.value);
            this.cdRef.markForCheck();
            return;
        }
        // This might happen if the user sets the value higher than the max (same behaviour in Chrome)
        if (this.decimal.value > this.max) {
            this.decimal.value = this.max;
            this.propagateChange(this.decimal.value);
            this.cdRef.markForCheck();
            return;
        }
        if (this.decimal.value > this.min) {
            // If an integer and greater or equal than min + 1
            if (this.decimal.decimalPlaces === 0 && this.decimal.value >= (this.min + this.bigStep)) {
                this.decimal.value -= this.bigStep; // subtract
            }
            else { // Float value or smaller than min + 1
                // Next closest value minus step: 0.6 -> 0.5 -> 0.25 ...
                const stepValue = this.decimal.toNearest(this.smallStep, Round.down);
                // If greater or equal to min then set the value to the new stepValue
                if (stepValue >= this.min) {
                    this.decimal.value = stepValue;
                }
                else {
                    this.decimal.value = this.min;
                }
            }
            this.propagateChange(this.decimal.value);
            this.cdRef.markForCheck();
        }
        else {
            // Select value if min was reached
            this.numberInput.nativeElement.select();
        }
    }
    add(once) {
        if (once) {
            this.increment();
        }
        else {
            this.startCount(() => this.increment());
        }
    }
    subtract(once) {
        if (once) {
            this.decrement();
        }
        else {
            this.startCount(() => this.decrement());
        }
    }
    startCount(callback) {
        callback();
        this.intervalHandler = setTimeout(() => {
            this.intervalHandler = setInterval(() => {
                callback();
            }, TIMEOUT);
        }, BUFFER_TIMEOUT);
    }
    stopCount() {
        if (this.intervalHandler) {
            clearInterval(this.intervalHandler);
            this.intervalHandler = null;
        }
    }
    mouseenter() {
        this.hover = true;
    }
    mouseleave() {
        this.hover = false;
    }
    focusInput() {
        this.focus = true;
    }
    changeInput() {
        this.propagateTouched();
    }
    blurInput() {
        this.checkValue();
        this.focus = false;
    }
    writeValue(value) {
        this.value = value;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(fn) {
        this.propagateTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    ngDoCheck() {
        if (this.ngControl) {
            this.updateErrorState();
        }
    }
}
SpinnerComponent.decorators = [
    { type: Component, args: [{
                selector: 'simple-spinner',
                exportAs: 'simpleSpinner',
                template: "<div [ngClass]=\"{\r\n    'simple-spinner-border': showBorder,\r\n    'simple-spinner-invalid': errorState,\r\n    'simple-spinner-rtl': direction === Direction.rightToLeft,\r\n    'simple-spinner-ltr': direction === Direction.leftToRight\r\n  }\" class=\"simple-spinner-wrapper\" (mouseenter)=\"mouseenter()\" (mouseleave)=\"mouseleave()\">\r\n  <div class=\"simple-spinner-input-wrapper\">\r\n    <div *ngIf=\"showPlaceholder\" class=\"simple-spinner-placeholder\">\r\n      <span>{{placeholder}}</span>\r\n    </div>\r\n    <div [ngClass]=\"{'simple-spinner-inactive': showFractionOverlay}\" class=\"simple-spinner-input\">\r\n      <input #numberInput type=\"number\" tabindex=\"{{tabIndex}}\"\r\n        [value]=\"value\"\r\n        (change)=\"changeInput()\"\r\n        (keydown)=\"filterKey($event)\"\r\n        (focus)=\"focusInput()\"\r\n        (blur)=\"blurInput()\"\r\n        step=\"any\"\r\n        autocomplete=\"off\"\r\n        [disabled]=\"disabled\">\r\n    </div>\r\n    <div *ngIf=\"showFractionOverlay\" class=\"simple-spinner-fraction-overlay\">\r\n      <span dir=\"ltr\">{{decimal.precedingStr | i18nNumber:i18nNumbers}}\r\n        <sup dir=\"ltr\">{{decimal.dividend | i18nNumber:i18nNumbers}}</sup>&frasl;<sub dir=\"ltr\">{{decimal.divisor | i18nNumber:i18nNumbers}}</sub>\r\n      </span>\r\n    </div>\r\n  </div>\r\n  <div *ngIf=\"showNav\" class=\"simple-spinner-nav\">\r\n    <button [disabled]=\"disabled\" class=\"simple-spinner-arrow simple-spinner-arrow-up\" (mousedown)=\"add()\" (mouseup)=\"stopCount()\" (mouseleave)=\"stopCount()\" tabindex=\"-1\"></button>\r\n    <button [disabled]=\"disabled\" class=\"simple-spinner-arrow simple-spinner-arrow-down\" (mousedown)=\"subtract()\" (mouseup)=\"stopCount()\" (mouseleave)=\"stopCount()\" tabindex=\"-1\"></button>\r\n  </div>\r\n</div>\r\n",
                inputs: ['disabled', 'tabIndex'],
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    '[attr.id]': 'id',
                    class: 'simple-spinner',
                    role: 'input',
                    '[attr.aria-label]': 'ariaLabel',
                    '[attr.aria-required]': 'required.toString()',
                    '[attr.aria-disabled]': 'disabled.toString()',
                    '[attr.aria-invalid]': 'errorState',
                    '[class.simple-disabled]': 'disabled',
                    '[class.simple-required]': 'required',
                    '(keydown)': '!disabled && keyManager.catchKey($event)'
                }
            },] }
];
SpinnerComponent.ctorParameters = () => [
    { type: DeviceService },
    { type: ElementRef },
    { type: ErrorStateMatcher },
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] },
    { type: ChangeDetectorRef },
    { type: LanguageService }
];
SpinnerComponent.propDecorators = {
    placeholder: [{ type: Input }],
    dir: [{ type: Input }],
    errorStateMatcher: [{ type: Input }],
    _ariaLabel: [{ type: Input, args: ['aria-label',] }],
    id: [{ type: Input }],
    value: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    hoverBorder: [{ type: Input }],
    hoverNav: [{ type: Input }],
    fractions: [{ type: Input }],
    i18nNumbers: [{ type: Input }],
    bigStep: [{ type: Input }],
    smallStep: [{ type: Input }],
    maxDecimalPlaces: [{ type: Input }],
    required: [{ type: Input }],
    numberInput: [{ type: ViewChild, args: ['numberInput', { static: true },] }]
};

class InternationalNumberPipe {
    constructor(langService) {
        switch (langService.preferredLanguage) {
            // Arabic-Indic
            case 'ar': // Arabic
                this._offset = 1584;
                break;
            // Eastern Arabic-Indic
            case 'fa': // Persian
            case 'ur': // Urdu
                this._offset = 1728;
                break;
            // Ignore Hindi, Thai, ... numerals, since they are rarely used
        }
    }
    transform(num, apply = true) {
        const numStr = String(num);
        // Condition whether or not to apply the transformation
        if (!apply) {
            return numStr;
        }
        /*
        European             U+0030..U+0039   0123456789
        Arabic-Indic         U+0660..U+0669   ٠١٢٣٤٥٦٧٨٩
        Eastern Arabic-Indic U+06F0..U+06F9   äÒÚÛÙıˆ˜¯˘
        */
        if (this._offset === undefined) {
            return numStr;
        }
        let transformedNumStr = '';
        // eslint-disable-next-line @typescript-eslint/prefer-for-of
        for (let i = 0; i < numStr.length; i++) {
            const charCode = numStr.charCodeAt(i);
            if (charCode < 48 || charCode > 57) {
                transformedNumStr += numStr[i];
                continue;
            }
            transformedNumStr += String.fromCharCode(charCode + this._offset);
        }
        return transformedNumStr;
    }
}
InternationalNumberPipe.decorators = [
    { type: Pipe, args: [{ name: 'i18nNumber' },] }
];
InternationalNumberPipe.ctorParameters = () => [
    { type: LanguageService }
];

class SimpleSpinnerModule {
}
SimpleSpinnerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [SpinnerComponent, InternationalNumberPipe],
                imports: [
                    CommonModule,
                    FormsModule
                ],
                providers: [
                    DeviceService
                ],
                exports: [SpinnerComponent, FormsModule, InternationalNumberPipe]
            },] }
];

/*
 * Public API Surface of simple-spinner
 */

/**
 * Generated bundle index. Do not edit.
 */

export { SimpleSpinnerModule, SpinnerComponent, DeviceService as ɵa, LanguageService as ɵb, InternationalNumberPipe as ɵc };
//# sourceMappingURL=nutrify-ngx-simple-spinner.js.map
